Checkpointing API

Use the checkpointing API to checkpoint jobs running under LoadLeveler.


This API consists of the following subroutines:


*  ll_ckpt subroutine
*  ll_migrate_job_step subroutine


For more information, see the "LoadLeveler support for checkpointing
jobs" topic in  LoadLeveler: Using and Administering. For information on
checkpointing parallel jobs, see IBM Parallel Environment Runtime
Edition for AIX and Linux: Operation and Use.



ll_ckpt subroutine


Use the ll_ckpt subroutine to initiate a checkpoint on a specific job
step.


Library


LoadLeveler API library libllapi.a or libllapi.so (Linux)


Syntax


#include "llapi.h"

int ll_ckpt (LL_ckpt_info *ckpt_info);


Parameters


ckpt_info
   A pointer to a LL_ckpt_info structure, which has the following
   fields: 

   int version
      The version of the API that the program was compiled with (from
      llapi.h).

   char* step_id
      A job or step identifier. When a job identifier is specified, the
      action of the API is taken for all steps of the job. At least one
      job or step identifier must be specified. 

      The format of a job identifier is host.jobid. The format of a step
      identifier is host.jobid.stepid.       

      where:
       
      *  host is the name of the machine that assigned the job and step
         identifiers.

      *  jobid is the job number assigned to the job when it was
         submitted.

      *  stepid is the job step number assigned to the job step when it
         was submitted.
       
      The job or step identifier can be specified in an abbreviated
      form, jobid or jobid.stepid, when the API is invoked on the same
      machine that assigned the job and step identifiers. In this case,
      LoadLeveler will use the local machine's host name to construct
      the full job or step identifier.

   enum ckpt_type ckptType
      The action to be taken after the checkpoint successfully
      completes. The values for enum ckpt_type are: 

      CKPT_AND_CONTINUE
               Allows the job to continue after the checkpoint.

      CKPT_AND_TERMINATE
               Terminates the job after the checkpoint.

      CKPT_AND_HOLD
               Puts the job on user hold after the checkpoint. A
               coschedule job step can be specified with the
               CKPT_AND_HOLD option.

      Note:    If checkpoint is not successful, the job continues on
               return regardless of these settings.

   enum wait_option waitType
      The flag used to identify blocking action during checkpoint. By
      default, ll_ckpt() will block until the checkpoint completes. The
      values for the enum wait_option are: 

      CKPT_NO_WAIT
               Disables blocking while the job is being checkpointed.

      CKPT_WAIT
               The job is blocked while being checkpointed. This is the
               default.

   int abort_sig
      Identifies the signal to be used to interrupt a checkpoint
      initiated by the API. Upon receipt of this signal the checkpoint
      will be aborted. Default is SIGINT.

   cr_error_t *cp_error_data
      The structure containing error information from the checkpoint
      operation.

   int ckpt_rc
      The return code from checkpoint.

   int soft_limit
      The time, in seconds, indicating the maximum time allocated for a
      checkpoint operation to complete before the checkpoint operation
      is aborted. The job is allowed to continue. The value for
      soft_limit specified here will override any soft limit value
      specified in the job command file. If the value for soft limit
      specified by the administration file is less than the value
      specified here, the administration file value takes precedence. 

      Values are: 

      -1       Indicates there is no limit.

      0        Indicates the existing soft limit for the job step should
               be enforced.

      Positive integer
               Indicates the number of seconds allocated for the limit.

   int hard_limit
      The time, in seconds, indicating the maximum time allocated for a
      checkpoint operation to complete before the job is terminated. The
      value for hard-limit specified here will override any hard limit
      value specified in the job command file. If the value for hard
      limit specified by the administration file is less than the value
      specified here, the administration file value will take
      precedence. 

      Values are: 

      -1       Indicates there is no limit.

      0        Indicates the existing hard limit for the job step should
               be enforced.

      Positive integer
               Indicates the number of seconds allocated for the limit.


Description


This function initiates a checkpoint for the specified job step.
ll_ckpt() will, by default, block until the checkpoint operation
completes. To disable blocking, the flag waitType must be set to
NO_WAIT. This function is allowed to be executed by the owner of the job
step or a LoadLeveler administrator.


Return Values


0  Checkpoint completed successfully.

1  Checkpoint event did not receive status and the success or failure of
   the checkpoint is unclear.


Error Values


-1 
   Error occurred attempting to checkpoint.

-2 
   Format not valid for job step, not in the form host.jobid.stepid.

-3 
   Cannot allocate memory.

-4 
   API cannot create listen socket.

-6 
   Configuration file errors.



ll_migrate_job_step subroutine


Use the ll_migrate_job_step subroutine in administrator-owned programs
to migrate tasks of a running job to a new set of nodes.


Library


LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)


Syntax


#include "llapi.h"

int ll_migrate_job_step (int version, LL_element **err_obj,
LL_migrate_param *migrate_param)


Parameters


version
   Is an input parameter that indicates the LoadLeveler API version
   (this should be the same value as LL_API_VERSION in llapi.h).

err_obj
   Provides the address of a pointer to LL_element that points to an
   error object if this function fails. The caller must free the error
   object storage before reusing the pointer. You can also use the
   ll_error subroutine to display error messages stored in the error
   object. If you are going to do so, initialize the pointer to NULL to
   avoid a segmentation fault when the pointer is passed to the ll_error
   subroutine.

migrate_param
   Is the structure that specifies the parameters to migrate the tasks
   of a running job step from one set of machines to another set of
   machines. 

   The ll_migrate_job_step structure contains the following fields: 

   step_id
      A pointer to a character string containing the ID of the job step
      to be migrated.

   wait
      An enum to indicate if the API function will wait until the
      migration is complete. Possible values are MIGRATE_NO_WAIT and
      MIGRATE_WAIT.

   messages
      A boolean to indicate if messages regarding the progress of the
      migration are to be written to stdout. Possible values are
      MIGRATE_MESSAGE_NONE and MIGRATE_MESSAGE_VERBOSE.

   from_list
      A pointer to an array of character string pointers that contain
      names of the machines from which the job step will be migrated.
      The number of entries in the array is machine_count.

   to_list
      A pointer to an array of character string pointers that contain
      names of the machines to which the job step will be migrated. The
      number of entries in the array is machine_count. This pointer is
      set to NULL if a reservation ID is specified. To specify that the
      scheduler should select the destination machines for the
      migration, set to_list and reservation_ID to NULL.

   machine_count
      An integer indicating the number of machines the job step is to be
      migrated from.

   reservation_id
      A pointer to a character string that contains an identifier that
      the scheduler can use to identify a set of machines reserved for
      this migration request. This pointer is set to NULL if a list of
      machines is specified. To specify that the scheduler should select
      the destination machines for the migration, set to_list and
      reservation_ID to NULL.

   migrate_rc
      A pointer to an array of integers that is used to return the
      migration return codes for each machine from which the job step is
      migrated. Each entry will contain the migration return code for
      the migration from the machine in the corresponding entry in the
      from_list array. The number of entries in the array is
      machine_count. The migration return codes are set only if the wait
      option is set to MIGRATE_WAIT.


Description


The ll_migrate_job_step interface is called to migrate parallel tasks
from one set of running nodes to a new set of nodes. The destination
nodes can be explicitly specified with a list of host names or an ID
that specifies that a set of resources previously reserved can be used.
If resources have not been allocated for the destination nodes, then the
scheduler will attempt to find resources for the migration. In all
cases, an event is sent to the scheduler to either select resources or
verify that the specified resource satisfy the requirements of the job
step.


Return Values


If the wait option was set to MIGRATE_NO_WAIT, then the following return
values are used: 

0        The migration request was successfully sent to LoadLeveler.

-1       The migration request could not be sent to LoadLeveler. Error
         messages can be obtained through the err_obj pointer.


Error Values


If the wait option was set to MIGRATE_WAIT, the following return values
are used:


0        The migration was successful on all machines

>0       The migration failed for one or more machines. The return value
         will be set to the number of machines that the migration failed
         on and the migrate_rc array will be set with a return code for
         each individual machine.

-1       An error occurs before the migration operation could be
         started. Error messages can be obtained through the err_obj
         pointer.

-2       The job step failed to resume after the migration operation
         completed. Error messages can be obtained through the err_obj
         pointer.