Configuration API

Use the configuration API to operate on LoadLeveler configuration files.


This API consists of the following subroutines:

*  ll_config subroutine
*  ll_config_changed subroutine
*  ll_read_config subroutine


ll_config subroutine


Use the ll_config subroutine to manage LoadLeveler's configuration
database. This subroutine provides options to display or change the
value of the LoadLeveler keywords when the database option is being
used.


Library


LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)


Syntax 

  
#include llapi.h
enum LL_config_op { CONFIG_C, CONFIG_D, CONFIG_I, CONFIG_STANZA_A, CONFIG_STANZA_C, 
                    CONFIG_STANZA_D, CONFIG_STANZA_F, CONFIG_STANZA_R };
     

int ll_config (int version, enum LL_config_op config_op, char **input,
               char ***output, char **hostlist,
               LL_element **err_obj,int no_reconfig);


Parameters


version
   Specifies the version of this API.

config_op
   Specifies the operation type: 

   CONFIG_C Changes the value of the specified keywords relating to
            machines.

   CONFIG_D Displays the value of the specified keywords relating to
            machines.

   CONFIG_I Reserved for LoadLeveler internal use.

   CONFIG_STANZA_A
            Adds a new stanza to the configuration database with no
            keyword values set.

   CONFIG_STANZA_C
            Changes the value of the specified keywords relating to
            machine, machine_group, user, group, class, region, and
            cluster.

   CONFIG_STANZA_D
            Displays the value of the specified keywords relating to
            machine, machine_group, user, group, class, region, and
            cluster.

   CONFIG_STANZA_F
            Adds new stanzas to the database from a specified file.

   CONFIG_STANZA_R
            Removes the specified stanza from the configuration
            database.

input
   A NULL-terminated char* array of input that depends on config_op. 

   CONFIG_C The elements of the array are strings in a format similar to
            keyword =value. They are used to change the value of any
            configuration keywords relating to machines. Keywords
            related to user, group, class, machine_group, and cluster
            cannot be changed using this command.

   CONFIG_D Each element of the array is one keyword whose value is to
            be displayed. This option shows the value of any
            configuration keywords relating to machines. Keywords
            related to user, group, class, machine_group, and cluster
            cannot be displayed using this option.

   CONFIG_STANZA_A
            The elements of the array are strings in a format similar to
            stanza=value[:substanza]. The stanza can be machine,
            machine_group, user, group, class, region, and cluster.

   CONFIG_STANZA_C
            The elements of the array are strings in a format similar to
            stanza=value[:substanza] keyword1=value1.... They are used
            to change the value of any configuration keywords relating
            to machine, machine_group, user, group, class, region, and
            cluster.

   CONFIG_STANZA_D
            The elements of the array are strings in a format similar to
            stanza[=value[:substanza]]. They are used to display the
            value of any configuration keywords relating to machine,
            machine_group, user, group, class, region, and cluster.

   CONFIG_STANZA_F
            The array can only have one element that specifies the
            stanza file that contains the new stanzas.

   CONFIG_STANZA_R
            The elements of the array are strings in a format similar to
            stanza=value[:substanza]. The stanza can be machine,
            machine_group, user, group, class, region, and cluster.

output
   A NULL-terminated char* array for CONFIG_D. Each element of the array
   is a string in a format similar to keyword=value and each string
   corresponds to one keyword from the input. It is the caller's
   responsibility to free the storage associated with this array. 

   This parameter is also used by CONFIG_STANZA_D. Each element of the
   array is a string in a format similar to how the stanza is configured
   in the administration file.

hostlist
   A NULL-terminated char* array of host names.

errObj
   Is the address of a pointer to a LoadLeveler error object. The
   pointer to the LL_element should be set to NULL before calling this
   function. If this function fails, the pointer will point to an error
   object. The error messages stored in the error object can be
   displayed through the ll_error function. The caller should free the
   error object storage before reusing the pointer. If this function
   fails with some keywords, but succeeds with others, the error
   messages for the failed keywords are stored in this object.


Description


This subroutine performs operations that are essentially equivalent to
that performed by the llconfig command with the -c or -d option.


Return Values


LLCONFIG_DB_CANNOT_CONNECT
         Cannot connect to the database.

LLCONFIG_CHECK_FAILED
         Configuration errors were found when checking the configuration
         files.

LLCONFIG_CHANGE_AUTH_FAILED
         The caller is not authorized to run this operation.

LLCONFIG_CHANGE_OK
         The values for all or some keywords were changed. It is
         possible that some keywords are not valid or that there is no
         related record in the database.

LLCONFIG_CHANGE_RECONFIG_ERROR
         The values for all or some keywords were changed, but the
         reconfiguration procedure cannot finish for some or all hosts.

LLCONFIG_NOT_VALID_INPUT
         An input parameter was specified that is not valid.

LLCONFIG_DISPLAY_OK
         The records for all or some keywords were found and displayed.

LLCONFIG_LOAD_ODBC_LIBRARY_FAILED 
         The unixODBC library cannot be loaded.

LLCONFIG_LOADLDB_NOT_CONFIG 
         The LoadLDB keyword is not specified in the master
         configuration file.

LLCONFIG_CONFIG_OPT_UNSUPPORTED
         A config_op was specified that is not valid.


Related Information


Commands: llconfig



ll_config_changed subroutine

Use the ll_config_changed subroutine to allow users to determine if the
LoadLeveler configuration has changed since the last time the
configuration data was read.


Library


LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)


Syntax 

  
#include "llapi.h"
int ll_config_changed(void);


Description


This subroutine is used in conjunction with the ll_read_config
subroutine to reread configuration data.


Return Values


An integer value will be returned that indicates whether the
configuration data was changed since the last time it was read.

0        The configuration data was not changed.

1        Configuration data was changed.


Related Information


Subroutine: ll_read_config



ll_read_config subroutine

Use the ll_read_config subroutine to allow users to read or reread
LoadLeveler configuration data.


Library


LoadLeveler API library libllapi.a (AIX) or libllapi.so (Linux)


Syntax 

  
#include "llapi.h"
int ll_read_config(LL_element **errObj);


Parameters


errObj
   This is the address of a pointer to a LoadLeveler error object. The
   pointer to the LL_element should be set to NULL before calling this
   function. If this function fails, the pointer will point to an error
   object. The error messages stored in the error object can be
   displayed through the ll_error function. The caller should free the
   error object storage before reusing the pointer.


Description


This subroutine is used in conjunction with the ll_config_changed
subroutine to determine whether the configuration data was changed since
the last time it was read.


Return Values


API_OK   The configuration data was successfully read.

API_CONFIG_ERR
         Errors were encountered while reading the configuration data.


Related Information


Subroutines: ll_config_changed, ll_error
