// The following code queries and displays information about all
// blocks in the Blue Gene system.

#include <stdio.h>
#include <stdlib.h>
#include "llapi.h"
#include "bg_common.h"

void  displayList(char ** );

int main(int argc, char *argv[]) {
	int rc;

	// Initialize the query Blue Gene
	LL_element *queryObject = ll_query(BLUE_GENE);
	if (!queryObject) {
		printf("Query Blue Gene: ll_query() returns NULL.\n");
		exit(1);
	}

	// Set the query request for all blocks
	char *block_list[2];
	block_list[0] = "all";
	block_list[1] = NULL;

	rc = ll_set_request(queryObject, QUERY_BG_BLOCK, block_list, ALL_DATA);
	if (rc) {
		printf("Query MACHINES: ll_set_request() return code = %d\n",
		       rc);
		exit(1);
	}

	// Get the Blue Gene machine object. Only one machine object is returned
	int err_code = 0;
	int obj_count = 0;
	LL_element *machine = ll_get_objs(queryObject, LL_CM, NULL, &obj_count, &err_code);
	if (machine == NULL || obj_count != 1) {
		printf("Query Blue Gene: ll_get_objs() error code = %d\n", err_code);
		exit(1);
	}

	// Get data from Blue Gene machine
	int blockCount; 
	rc = ll_get_data(machine, LL_BgMachineBlockCount, &blockCount);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineBlockCount rc = %d\n",rc);
		exit(1);
	}
	printf("The number of Blocks defined in the Blue Gene System = %d.\n\n", blockCount);

	//
	// Blocks
	//
	LL_element *block; 
	rc = ll_get_data(machine, LL_BgMachineGetFirstBlock, &block);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineGetFirstBlock rc = %d\n",rc);
		exit(1);
	}

	while(block != NULL) {
		// get block name
		char *blockName;
		rc = ll_get_data(block, LL_BgBlockName, &blockName);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockName rc = %d\n",rc);
			exit(1);
		}

		// get block status 
		int blockStatus;
		rc = ll_get_data(block, LL_BgBlockStatus, &blockStatus);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockStatus rc = %d\n",rc);
			exit(1);
		}

		// get boolean value for small block
		int smallBlock;
		rc = ll_get_data(block, LL_BgBlockSmall, &smallBlock);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockSmall rc = %d\n",rc);
			exit(1);
		}

		// get block size 
		int blockSize;
		rc = ll_get_data(block, LL_BgBlockSize, &blockSize);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockSize rc = %d\n",rc);
			exit(1);
		}

		// get block shape
		int *blockShape = NULL;
		rc = ll_get_data(block, LL_BgBlockShape, &blockShape);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockShape rc = %d\n",rc);
			exit(1);
		}

		// Get midplanes 
		int mpCount; 
		rc = ll_get_data(block, LL_BgBlockMPCount, &mpCount);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockMPCount rc = %d\n",rc);
			exit(1);
		}

		char **midplaneList;
		if (mpCount > 0) {
			rc = ll_get_data(block, LL_BgBlockMPList, &midplaneList);
			if (rc) {
				printf("Error: ll_get_data LL_BgBlockMPList rc = %d\n",rc);
				exit(1);
			}
		}

		// Get passthrough Pidplanes 
		int passthruCount; 
		rc = ll_get_data(block, LL_BgBlockPassthroughCount, &passthruCount);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockPassthroughCount rc = %d\n",rc);
			exit(1);
		}

		char **passthruList;
		if (passthruCount> 0) {
			rc = ll_get_data(block, LL_BgBlockPassthroughList, &passthruList);
			if (rc) {
				printf("Error: ll_get_data LL_BgBlockPassthroughList rc = %d\n",rc);
				exit(1);
			}
		}

		// Get Cables
		int cableCount;
		rc = ll_get_data(block, LL_BgBlockCableCount, &cableCount);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockCableCount rc = %d\n",rc);
			exit(1);
		}

		// Get Nodeboards
		int nodeboardCount;
		rc == ll_get_data(block, LL_BgBlockNodeBoardCount, &nodeboardCount);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockNodeBoardCount rc = %d\n",rc);
			exit(1);
		}

		char **nodeboardList;
		if (nodeboardCount > 0) {
			rc = ll_get_data(block, LL_BgBlockNodeBoardList, &nodeboardList);
			if (rc) {
				printf("Error: ll_get_data LL_BgBlockNodeBoardList rc = %d\n",rc);
				exit(1);
			}
		}

		// Get IOLinks 
		int iolinkCount;
		rc == ll_get_data(block, LL_BgBlockIOLinkCount, &iolinkCount);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockIOLinkCount rc = %d\n",rc);
			exit(1);
		}

		char **iolinkList;
		if (iolinkCount > 0) {
			rc = ll_get_data(block, LL_BgBlockIOLinkList, &iolinkList);
			if (rc) {
				printf("Error: ll_get_data LL_BgBlockIOLinkList rc = %d\n",rc);
				exit(1);
			}
		}

		// get block owner 
		char *blockOwner;
		rc = ll_get_data(block, LL_BgBlockOwner, &blockOwner);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockOwner rc = %d\n",rc);
			exit(1);
		}

		// get block connectivity 
		int *blockConnectivity;
		rc = ll_get_data(block, LL_BgBlockConnectivity, &blockConnectivity);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockConnectivity rc = %d\n",rc);
			exit(1);
		}

		// get block description
		char *blockDescription;
		rc = ll_get_data(block, LL_BgBlockDescription, &blockDescription);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockDescription rc = %d\n",rc);
			exit(1);
		}

		// get block micro loader 
		char *blockMLoader;
		rc = ll_get_data(block, LL_BgBlockMLoaderImage, &blockMLoader);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockMLoaderImage rc = %d\n",rc);
			exit(1);
		}

		// get block node configuration
		char *blockNodeConfiguration;
		rc = ll_get_data(block, LL_BgBlockNodeConfiguration, &blockNodeConfiguration);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockNodeConfiguration rc = %d\n",rc);
			exit(1);
		}

		// get block user count
		int blockUserCount;
		rc = ll_get_data(block, LL_BgBlockUserCount, &blockUserCount);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockUserCount rc = %d\n",rc);
			exit(1);
		}

		// get block users 
		char **userList;
		if (blockUserCount > 0) {
			rc = ll_get_data(block, LL_BgBlockUserList, &userList);
			if (rc) {
				printf("Error: ll_get_data LL_BgBlockUserList rc = %d\n",rc);
				exit(1);
			}
		}
		
		printf("Block Name: %s\n", blockName); 
		printf("   Status: %s\n", getBlockStatusString(blockStatus).c_str());
		printf("   Size: %d\n", blockSize);
		printf("   Small Block: %s\n", smallBlock?"True":"False");
		if (smallBlock) {
			printf("   Shape: %dx%dx%dx%dx%d\n", blockShape[0], blockShape[1], blockShape[2], blockShape[3], blockShape[4]);
			printf("   Connectivity: N/A\n"); 
		}
		else  {
			printf("   Shape: %dx%dx%dx%d\n", blockShape[0], blockShape[1], blockShape[2], blockShape[3]);
			printf("   Connectivity: %s %s %s %s\n", getConnectivityString(blockConnectivity[0]).c_str(), getConnectivityString(blockConnectivity[1]).c_str(),
								 getConnectivityString(blockConnectivity[2]).c_str(), getConnectivityString(blockConnectivity[3]).c_str()); 
		}
		printf("   Owner: %s\n", blockOwner);
		printf("   Description: %s\n", blockDescription);
		printf("   Micro Loader: %s\n", blockMLoader);
		printf("   Node Configuration: %s\n", blockNodeConfiguration);
		if (blockUserCount > 0) {
			printf("   User Count: %d ( ", blockUserCount);
			displayList(userList);
				fflush(stdout);
			printf(")\n");
			free(userList); 
		}
		else {
			printf("   User Count: %d\n", blockUserCount);
		}


		if(!smallBlock) {
			// print midplanes and passthrough for large blocks
			if (mpCount > 0) {
				printf("   Midplane list = ");
				displayList(midplaneList);
				fflush(stdout);
				free(midplaneList);
			} else {
				printf("   No midplanes being used by the block."); 
			}
			printf("\n");
	
			if (passthruCount > 0) { 
				printf("   Passthrough list = ");
				displayList(passthruList);
				fflush(stdout);
				free(passthruList);
			} else {
				printf("   No passthrough midplanes being used by the block."); 
			}
			printf("\n");

		} else {
			// print nodeboards for small blocks
			if (nodeboardCount > 0) {
				printf("   Nodeboard list = ");
				displayList(nodeboardList);
				fflush(stdout);
				free(nodeboardList);
			} else {
				printf("   No nodeboards being used by the block.\n"); 
			}
			printf("\n");

			// print iolinks being used by small block
			if (iolinkCount > 0) { 
				printf("   IO link list = ");
				displayList(iolinkList);
				fflush(stdout);
				free(iolinkList);
			} 
			printf("\n");
		}

		printf("\n");

		int switchCount;
		rc = ll_get_data(block, LL_BgBlockSwitchCount, &switchCount);
		if (rc) {
			printf("Error: ll_get_data LL_BgPartitionSwitchCount rc = %d\n",rc);
			exit(1);
		}

		// Switches assigned to the Block
		LL_element *swt;
		rc = ll_get_data(block, LL_BgBlockGetFirstSwitch, &swt);
		if (rc) {
			printf("Error: ll_get_data LL_BgBlockGetFirstSwitch rc = %d\n",rc);
			exit(1);
		}

		while(swt != NULL) {
			char *switchLocation;
			rc = ll_get_data(swt, LL_BgSwitchLocation, &switchLocation);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchLocation rc = %d\n",rc);
				exit(1);
			}

			int switchState;
			rc = ll_get_data(swt, LL_BgSwitchState, &switchState);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchState rc = %d\n",rc);
				exit(1);
			}

			char *switchMPLocation;
			rc = ll_get_data(swt, LL_BgSwitchMidplaneLocation, &switchMPLocation);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchMidplaneLocation rc = %d\n",rc);
				exit(1);
			}

			int switchInUse;
			rc = ll_get_data(swt, LL_BgSwitchInUse, &switchInUse);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchInUserc = %d\n",rc);
				exit(1);
			}

			int switchDimension;
			rc = ll_get_data(swt, LL_BgSwitchDimension, &switchDimension);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchDimension rc = %d\n",rc);
				exit(1);
			}

			int switchSettingCount;
			rc = ll_get_data(swt, LL_BgSwitchSwitchSettingCount, &switchSettingCount);
			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchSwitchSettingCount rc = %d\n",rc);
				exit(1);
			}

			printf("   Switch=%s State=%s MP=%s InUse=%s Dim=%s\n",
				switchLocation, getHardwareStateString(switchState).c_str(), switchMPLocation,
				getSwitchInUseString(switchInUse).c_str(), getDimensionString(switchDimension).c_str());

			free(switchLocation);
			free(switchMPLocation); 

			//
			// Switch Settings
			//
			if (switchSettingCount > 0) { 
				LL_element *switchsetting;
				rc = ll_get_data(swt, LL_BgSwitchGetFirstSwitchSetting, &switchsetting);
				if (rc) {
					printf("Error: ll_get_data LL_BgSwitchGetFirstSwitchSetting rc = %d\n",rc);
					exit(1);
				}

				while(switchsetting != NULL) {
					char *ssLocation;
					rc = ll_get_data(switchsetting, LL_BgSwitchSettingLocation, &ssLocation);
					if (rc) {
						printf("Error: ll_get_data LL_BgSwitchSettingLocation rc = %d\n",rc);
						exit(1);
					}

					int ssPortSetting; 
					rc = ll_get_data(switchsetting, LL_BgSwitchSettingPortSetting, &ssPortSetting);
					if (rc) {
						printf("Error: ll_get_data LL_BgSwitchSettingPortSetting rc = %d\n",rc);
						exit(1);
					}
	
					int ssIncludeMidplane; 
					rc = ll_get_data(switchsetting, LL_BgSwitchSettingIsIncludedMidplane, &ssIncludeMidplane);
					if (rc) {
						printf("Error: ll_get_data LL_BgSwitchSettingIsIncludedMidplane rc = %d\n",rc);
						exit(1);
					}
	
					char *ssComputeBlock; 
					rc = ll_get_data(switchsetting, LL_BgSwitchSettingComputeBlock, &ssComputeBlock);
					if (rc) {
						printf("Error: ll_get_data LL_BgSwitchSettingComputeBlock rc = %d\n",rc);
						exit(1);
					}
	
					int ssIsPassthru; 
					rc = ll_get_data(switchsetting, LL_BgSwitchSettingIsPassthrough, &ssIsPassthru);
					if (rc) {
						printf("Error: ll_get_data LL_BgSwitchSettingIsPassthrough rc = %d\n",rc);
						exit(1);
					}
	
					printf("      SwitchSetting=%s PortSetting=%s IncludesMidplane=%s ComputeBlock=%s isPassThrough=%s\n",
						ssLocation, getPortSettingString(ssPortSetting).c_str(), ssIncludeMidplane?"True":"False", ssComputeBlock, ssIsPassthru?"True":"False");
	
					free(ssLocation);
					free(ssComputeBlock); 
	
					rc = ll_get_data(swt, LL_BgSwitchGetNextSwitchSetting, &switchsetting);
					if (rc) {
						printf("Error: ll_get_data LL_BgSwitchGetNextSwitchSetting rc = %d\n",rc);
						exit(1);
					}
				}
			}

			rc = ll_get_data(block, LL_BgBlockGetNextSwitch, &swt);
			if (rc) {
				printf("Error: ll_get_data LL_BgBlockGetNextSwitch rc = %d\n",rc);
				exit(1);
			}
		}
	
		free (blockName);
		free (blockOwner);

		rc = ll_get_data(machine, LL_BgMachineGetNextBlock, &block);
		if (rc) {
			printf("Error: ll_get_data LL_BgMachineGetNextBlock rc = %d\n",rc);
			exit(1);
		}
		printf("\n");
	}

	// Free objects obtained from Negotiator
	ll_free_objs(queryObject);
	// Free query element
	ll_deallocate(queryObject);

	return rc;
}

void displayList(char **ids) {
	int i;

	if (ids == NULL) return;
	for (i=0; ids[i]; i++ ) {
		printf("%s ", ids[i]);
	}
	return;
}
