// The following code queries and displays information about
// the Blue Gene machine.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "llapi.h"
#include "bg_common.h"

int main(int argc, char *argv[]) {
	int rc;

	// Initialize the query Blue Gene
	LL_element *queryObject = ll_query(BLUE_GENE);
	if (!queryObject) {
		printf("Query Blue Gene: ll_query() returns NULL.\n");
		 exit(1);
	}

	// Set query parameters to query all Blue Gene machine data
	rc = ll_set_request(queryObject, QUERY_ALL, NULL, ALL_DATA);
	if (rc) {
		printf("Query Blue Gene: ll_set_request() return code = %d\n", rc);
		exit(1);
	}

	// Get the Blue Gene machine object. Only one machine object is returned
	int err_code = 0;
	int obj_count = 0;
	LL_element *machine = ll_get_objs(queryObject, LL_CM, NULL, &obj_count, &err_code);
	if (machine == NULL || obj_count != 1) {
		printf("Query Blue Gene: ll_get_objs() error code = %d\n", err_code);
		exit(1);
	}

	// Get data from Blue Gene machine
	int *mach_size;
	rc = ll_get_data(machine, LL_BgMachineSize, &mach_size);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineSize rc = %d\n",rc);
		exit(1);
	}
	printf("Blue Gene Machine Size. . . . . . . %dx%dx%dx%d\n", mach_size[0],mach_size[1],mach_size[2],mach_size[3]);
	free(mach_size);
	
	int *mp_size;
	rc = ll_get_data(machine, LL_BgMachineMPSize, &mp_size);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineMPSize rc = %d\n",rc);
		exit(1);
	}
	printf("Blue Gene Midplane Size . . . . . . %dx%dx%dx%dx%d\n", mp_size[0], mp_size[1],mp_size[2],mp_size[3],mp_size[4]);
	free(mp_size);

	int switchCount;
	rc = ll_get_data(machine, LL_BgMachineSwitchCount, &switchCount);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineSwitchCount rc = %d\n",rc);
		exit(1);
	}
	printf("Blue Gene Machine Switch Count. . . %d\n", switchCount);

	int cableCount;
	rc = ll_get_data(machine, LL_BgMachineCableCount, &cableCount);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineCableCount rc = %d\n",rc);
		exit(1);
	}
	printf("Blue Gene Machine Cable Count . . . %d\n", cableCount);

	printf("\n");


	//
	// Midplanes
	//
	LL_element *mp;
	rc = ll_get_data(machine, LL_BgMachineGetFirstMP, &mp);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineGetFirstMP rc = %d\n",rc);
		exit(1);
	}

	while(mp != NULL) {
		char  *location;
		rc = ll_get_data(mp, LL_BgMPLocation, &location);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPLocation rc = %d\n",rc);
			exit(1);
		}

		int mpState;
		rc = ll_get_data(mp, LL_BgMPState, &mpState);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPStaterc = %d\n",rc);
			exit(1);
		}

		int  *coord;
		rc = ll_get_data(mp, LL_BgMPCoordinates, &coord);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPCoordinates rc = %d\n",rc);
			exit(1);
		}

		int   subDividedBusy;
		rc = ll_get_data(mp, LL_BgMPSubDividedBusy, &subDividedBusy);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPSubDividedBusy rc = %d\n",rc);
			exit(1);
		}


		printf("Midplane=%s (%d,%d,%d,%d) State=%s SubDivided=%s\n", 
			location, coord[0], coord[1], coord[2], coord[3], getHardwareStateString(mpState).c_str(), subDividedBusy?"True":"False");

		free(location); 

		
		rc = ll_get_data(machine, LL_BgMachineGetNextMP, &mp);
		if (rc) {
			printf("Error: ll_get_data LL_BgMachineGetNextMP rc = %d\n",rc);
			exit(1);
		}
	} // get next midplane 

	printf("\n");

	//
	// Switches
	//
	LL_element *swt;
	rc = ll_get_data(machine, LL_BgMachineGetFirstSwitch, &swt);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineGetFirstSwitch rc = %d\n",rc);
		exit(1);
	}

	while(swt != NULL) {
		char *switchLocation; 
		rc = ll_get_data(swt, LL_BgSwitchLocation, &switchLocation);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchLocation rc = %d\n",rc);
			exit(1);
		}

		int switchState;
		rc = ll_get_data(swt, LL_BgSwitchState, &switchState);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchState rc = %d\n",rc);
			exit(1);
		}

		char *switchMPLocation; 
		rc = ll_get_data(swt, LL_BgSwitchMidplaneLocation, &switchMPLocation);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchMidplaneLocation rc = %d\n",rc);
			exit(1);
		}

		int switchInUse;
		rc = ll_get_data(swt, LL_BgSwitchInUse, &switchInUse);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchInUse rc = %d\n",rc);
			exit(1);
		}

		int switchDimension; 
		rc = ll_get_data(swt, LL_BgSwitchDimension, &switchDimension);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchDimension rc = %d\n",rc);
			exit(1);
		}

		int switchSettingCount;
		rc = ll_get_data(swt, LL_BgSwitchSwitchSettingCount, &switchSettingCount);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchSwitchSettingCount rc = %d\n",rc);
			exit(1);
		}

		printf("Switch=%s (%s) MP=%s InUse=%s Dimension=%s\n", 
			switchLocation, getHardwareStateString(switchState).c_str(), switchMPLocation,
			getSwitchInUseString(switchInUse).c_str(), getDimensionString(switchDimension).c_str());

		free(switchLocation);
		free(switchMPLocation);

		// 
		// Switch Settings 
		//
		if (switchSettingCount > 0) { 
			LL_element *switchSetting;
			rc = ll_get_data(swt, LL_BgSwitchGetFirstSwitchSetting, &switchSetting);
			if (rc) { 
				printf("Error: ll_get_data LL_BgSwitchGetFirstSwitchSetting rc = %d\n", rc);
				exit(1);
			}

			while (switchSetting != NULL) { 
				char *switchSettingLocation; 
				rc = ll_get_data(switchSetting, LL_BgSwitchSettingLocation, &switchSettingLocation);
				if (rc) {
					printf("Error: ll_get_data LL_BgSwitchSettingLocation rc = %d\n",rc);
					exit(1);
				}

				int switchSettingPortSetting;
				rc = ll_get_data(switchSetting, LL_BgSwitchSettingPortSetting, &switchSettingPortSetting);
				if (rc) {
					printf("Error: ll_get_data LL_BgSwitchSettingPortSetting rc = %d\n",rc);
					exit(1);
				}

				int switchSettingIsIncludeMidplane; 
				rc = ll_get_data(switchSetting, LL_BgSwitchSettingIsIncludedMidplane, &switchSettingIsIncludeMidplane);
				if (rc) {
					printf("Error: ll_get_data LL_BgSwitchSettingIsIncludedMidplane rc = %d\n",rc);
					exit(1);
				}

				char *switchSettingComputeBlock; 
				rc = ll_get_data(switchSetting, LL_BgSwitchSettingComputeBlock, &switchSettingComputeBlock);
				if (rc) {
					printf("Error: ll_get_data LL_BgSwitchSettingComputeBlock rc = %d\n",rc);
					exit(1);
				}
			
				int switchSettingIsPassthrough;
				rc = ll_get_data(switchSetting, LL_BgSwitchSettingIsPassthrough, &switchSettingIsPassthrough);
				if (rc) {
					printf("Error: ll_get_data LL_BgSwitchSettingIsPassthrough rc = %d\n",rc);
					exit(1);
				}

				printf("\tSwitchSetting=%s\n", switchSettingLocation);
				printf("\t  PortSetting=%d\n", switchSettingPortSetting);
				printf("\t  IncludeMidplane=%s\n", switchSettingIsIncludeMidplane?"True":"False");
				printf("\t  ComputeBlock=%s\n", switchSettingComputeBlock);
				printf("\t  IsPassthrough=%s\n", switchSettingIsPassthrough?"True":"False");
	
				rc = ll_get_data(swt, LL_BgSwitchGetNextSwitchSetting, &switchSetting);
				if (rc) {
					printf("Error: ll_get_data LL_BgSwitchGetFirstSwitchSetting rc = %d\n", rc);
					exit(1);
				}
			}
		}

		rc = ll_get_data(machine, LL_BgMachineGetNextSwitch, &swt);
		if (rc) {
			printf("Error: ll_get_data LL_BgMachineGetNextSwitch rc = %d\n",rc);
			exit(1);
		}
	}
	printf("\n");


	//
	// Cables
	//
	LL_element *cable;
	rc = ll_get_data(machine, LL_BgMachineGetFirstCable, &cable);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineGetFirstCable rc = %d\n",rc);
		exit(1);
	}

	while(cable!= NULL) {
		char *cableLocation;
		rc = ll_get_data(cable, LL_BgCableLocation, &cableLocation);
		if (rc) {
			printf("Error: ll_get_data LL_BgCableLocation rc = %d\n",rc);
			exit(1);
		}

		int cableState;
		rc = ll_get_data(cable, LL_BgCableState, &cableState);
		if (rc) {
			printf("Error: ll_get_data LL_BgCableState rc = %d\n",rc);
			exit(1);
		}

		char *cableDestinationLocation;
		rc = ll_get_data(cable, LL_BgCableDestinationLocation, &cableDestinationLocation);
		if (rc) {
			printf("Error: ll_get_data LL_BgCableDestinationLocation rc = %d\n",rc);
			exit(1);
		}

		char *cableComputeBlock;
		rc = ll_get_data(cable, LL_BgCableComputeBlock, &cableComputeBlock);
		if (rc) {
			printf("Error: ll_get_data LL_BgCableComputeBlock rc = %d\n",rc);
			exit(1);
		}

		const char *block; 
		if (strlen(cableComputeBlock) == 0) {
			block="NONE";
		} else {
			block=cableComputeBlock;
		}

		printf("Cable location=%s (%s) destination=%s computeBlock=%s\n", 
			cableLocation, getHardwareStateString(cableState).c_str(), cableDestinationLocation, block); 

		free(cableLocation);
		free(cableDestinationLocation);
		free(cableComputeBlock);

		rc = ll_get_data(machine, LL_BgMachineGetNextCable, &cable);
		if (rc) {
			printf("Error: ll_get_data LL_BgMachineGetNextCable rc = %d\n",rc);
			exit(1);
		}
	}

	// Free objects obtained from Negotiator
	ll_free_objs(queryObject);
	// Free query element
	ll_deallocate(queryObject);
}
