// The following code queries and displays information about all midplanes
// in the Blue Gene system.

#include <stdio.h>
#include <stdlib.h>
#include <string> 
#include <string.h> 
#include <iostream>
#include "llapi.h"
#include "bg_common.h"

int main(int argc, char *argv[]) {
	int rc;

	// Initialize the query Blue Gene
	LL_element *queryObject = ll_query(BLUE_GENE);
	if (!queryObject) {
		printf("Query Blue Gene: ll_query() returns NULL.\n");
		exit(1);
	}

	// Set the query request for all midplanes
	char *mp_list[2];
	mp_list[0] = "All";
	mp_list[1] = NULL;

	rc = ll_set_request(queryObject, QUERY_BG_MIDPLANE, mp_list, ALL_DATA);
	if (rc) {
		printf("Query MACHINES: ll_set_request() return code = %d\n", rc);
		exit(1);
	}

	// Get the Blue Gene machine object. Only one machine object is returned
	int err_code = 0;
	int obj_count = 0;
	LL_element *machine = ll_get_objs(queryObject, LL_CM, NULL, &obj_count, &err_code);
	if (machine == NULL || obj_count != 1) {
		printf("Query Blue Gene: ll_get_objs() error code = %d\n", err_code);
		exit(1);
	}

	// Get data from Blue Gene machine
	int *mach_size;
	rc = ll_get_data(machine, LL_BgMachineSize, &mach_size);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineSize rc = %d\n",rc);
		exit(1);
	}
	printf("Blue Gene Machine Size = %dx%dx%dx%d\n", mach_size[0],mach_size[1],mach_size[2],mach_size[3]);
	free(mach_size);
	
	int *mp_size;
	rc = ll_get_data(machine, LL_BgMachineMPSize, &mp_size);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineMPSize rc = %d\n",rc);
		exit(1);
	}
	printf("Blue Gene Midplane Size = %dx%dx%dx%dx%d\n", mp_size[0], mp_size[1],mp_size[2],mp_size[3],mp_size[4]);
	free(mp_size);

	printf("\n");

	//
	// Midplanes
	//
	LL_element *mp;
	rc = ll_get_data(machine, LL_BgMachineGetFirstMP, &mp);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineGetFirstMP rc = %d\n",rc);
		exit(1);
	}

	while(mp != NULL) {
		char  *location;
		rc = ll_get_data(mp, LL_BgMPLocation, &location);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPLocation rc = %d\n",rc);
			exit(1);
		}

		int state;
		rc = ll_get_data(mp, LL_BgMPState, &state);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPStaterc = %d\n",rc);
			exit(1);
		}

		int  *coord;
		rc = ll_get_data(mp, LL_BgMPCoordinates, &coord);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPCoordinates rc = %d\n",rc);
			exit(1);
		}

		int   subDividedBusy;
		rc = ll_get_data(mp, LL_BgMPSubDividedBusy, &subDividedBusy);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPSubDividedBusy rc = %d\n",rc);
			exit(1);
		}

		char *computeBlock;
		rc = ll_get_data(mp, LL_BgMPComputeBlock, &computeBlock);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPComputeBlock rc = %d\n",rc);
			exit(1);
		}

		int nodeBoardCount;
		rc = ll_get_data(mp, LL_BgMPNodeBoardCount, &nodeBoardCount);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPNodeBoardCount rc = %d\n",rc);
			exit(1);
		}

		int cnodeMemory;
		rc = ll_get_data(mp, LL_BgMPCnodeMemory, &cnodeMemory);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPCnodeMemory rc = %d\n",rc);
			exit(1);
		}

		int iolinkCount; 
		rc = ll_get_data(mp, LL_BgMPIOLinkCount, &iolinkCount);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPIOLinkCount rc = %d\n",rc);
			exit(1);
		}

		const char *block;
		if (strlen(computeBlock) == 0) { 
			block="NONE";
		} else {
			block=computeBlock;
		}

		printf("Midplane=%s (%d,%d,%d,%d)\n   State=%s Memory=%d SubDivided=%s NodeboardCount=%d IOLinkCount=%d ComputeBlock=%s\n\n", 
			location, coord[0], coord[1], coord[2], coord[3], getHardwareStateString(state).c_str(), cnodeMemory,
			subDividedBusy?"True":"False", nodeBoardCount, iolinkCount, block);

		free(location); 
		free(computeBlock);


		//
		// Get Nodeboards:
		//
		LL_element  *nodeboard;
		rc = ll_get_data(mp, LL_BgMPGetFirstNodeBoard, &nodeboard);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPGetFirstNodeBoard rc = %d\n",rc);
			exit(1);
		}

		while(nodeboard != NULL) {
			char *nodeboardLocation; 
			rc = ll_get_data(nodeboard, LL_BgNodeBoardLocation, &nodeboardLocation);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeBoardLocationrc = %d\n",rc);
				exit(1);
			}

			int nodeboardState; 
			rc = ll_get_data(nodeboard, LL_BgNodeBoardState, &nodeboardState);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeBoardState rc = %d\n",rc);
				exit(1);
			}

			int nodeboardQuadrant;
			rc = ll_get_data(nodeboard, LL_BgNodeBoardQuadrant, &nodeboardQuadrant);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeBoardQuadrant rc = %d\n",rc);
				exit(1);
			}

			char *nodeboardComputeBlock;
			rc = ll_get_data(nodeboard, LL_BgNodeBoardComputeBlock, &nodeboardComputeBlock);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeBoardComputeBlock rc = %d\n",rc);
				exit(1);
			}

			int nodeboardInUse;
			rc = ll_get_data(nodeboard, LL_BgNodeBoardIsInUse, &nodeboardInUse);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeBoardIsInUse rc = %d\n",rc);
				exit(1);
			}

			int nodeboardIOLinkCount; 
			rc = ll_get_data(nodeboard, LL_BgNodeBoardIOLinkCount, &nodeboardIOLinkCount);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeBoardIOLinkCount rc = %d\n",rc);
				exit(1);
			}

			if (strlen(nodeboardComputeBlock) == 0) {
				block="NONE";
			} else {
				block=nodeboardComputeBlock;
			}

			printf("\tNodeboard=%s State=%s Quadrant=%s InUse=%d iolinkCount=%d ComputeBlock=%s\n", 
				nodeboardLocation, getHardwareStateString(nodeboardState).c_str(), getQuadrantString(nodeboardQuadrant).c_str(), 
				nodeboardInUse, nodeboardIOLinkCount, block);

			free(nodeboardLocation);
			free(nodeboardComputeBlock); 

			// IOLinks in BG/Q are part of the midplanes.  LoadLeveler associates the iolinks with the nodeboards where they reside. 
			// Umcomment to display iolinks on the nodeboards 
			/*
			LL_element  *ion;
			rc = ll_get_data(nodeboard, LL_BgNodeBoardGetFirstIOLink, &ion);
			if (rc) {
				printf("Error: ll_get_data LL_BgNodeBoardGetFirstIOLink rc = %d\n",rc);
				exit(1);
			}

			while(ion != NULL) {
				char *iolinkLocation;
				rc = ll_get_data(ion, LL_BgIOLinkLocation, &iolinkLocation);
				if (rc) {
					printf("Error: ll_get_data LL_BgIOLinkLocation rc = %d\n",rc);
					exit(1);
				}

				int iolinkState;
				rc = ll_get_data(ion, LL_BgIOLinkState, &iolinkState);
				if (rc) {
					printf("Error: ll_get_data LL_BgIOLinkState rc = %d\n",rc);
					exit(1);
				}

				char *iolinkDestination;
				rc = ll_get_data(ion, LL_BgIOLinkDestinationLocation, &iolinkDestination);
				if (rc) {
					printf("Error: ll_get_data LL_BgIOLinkDestinationLocation rc = %d\n",rc);
					exit(1);
				}

				char *iolinkMidplane;
				rc = ll_get_data(ion, LL_BgIOLinkMidplaneLocation, &iolinkMidplane);
				if (rc) {
					printf("Error: ll_get_data LL_BgIOLinkMidplaneLocation rc = %d\n",rc);
					exit(1);
				}

				printf("\t   -IOLink=%s State=%s Destination=%s Midplane=%s\n",
					iolinkLocation, getHardwareStateString(iolinkState).c_str(), iolinkDestination, iolinkMidplane);

				free(iolinkLocation);
				free(iolinkDestination);
				free(iolinkMidplane);

				rc = ll_get_data(nodeboard, LL_BgNodeBoardGetNextIOLink, &ion);
				if (rc) {
					printf("Error: ll_get_data LL_BgNodeBoardGetNextIOLink rc = %d\n",rc);
					exit(1);
				}
			}  // get iolinks on nodeboards 
			*/ 

			rc = ll_get_data(mp, LL_BgMPGetNextNodeBoard, &nodeboard);
			if (rc) {
				printf("Error: ll_get_data LL_BgMPGetNextNodeBoardrc = %d\n",rc);
				exit(1);
			}

		} // nodeboards


		//
		// Get IOLinks on the midplane  
		//
		LL_element  *iol;
		rc = ll_get_data(mp, LL_BgMPGetFirstIOLink, &iol);
		if (rc) {
			printf("Error: ll_get_data LL_BgMPGetFirstIOLink rc = %d\n",rc);
			exit(1);
		}

		while (iol != NULL) { 
			char *iolinkLocation;
			rc = ll_get_data(iol, LL_BgIOLinkLocation, &iolinkLocation);
			if (rc) {
				printf("Error: ll_get_data LL_BgIOLinkLocation rc = %d\n",rc);
				exit(1);
			}

			int iolinkState;
			rc = ll_get_data(iol, LL_BgIOLinkState, &iolinkState);
			if (rc) {
				printf("Error: ll_get_data LL_BgIOLinkState rc = %d\n",rc);
				exit(1);
			}

			char *iolinkDestination;
			rc = ll_get_data(iol, LL_BgIOLinkDestinationLocation, &iolinkDestination);
			if (rc) {
				printf("Error: ll_get_data LL_BgIOLinkDestinationLocation rc = %d\n",rc);
				exit(1);
			}

			char *iolinkMidplane;
			rc = ll_get_data(iol, LL_BgIOLinkMidplaneLocation, &iolinkMidplane);
			if (rc) {
				printf("Error: ll_get_data LL_BgIOLinkMidplaneLocation rc = %d\n",rc);
				exit(1);
			}

			printf("\tIOLink=%s State=%s Destination=%s Midplane=%s\n",
				iolinkLocation, getHardwareStateString(iolinkState).c_str(), iolinkDestination, iolinkMidplane);

			free(iolinkLocation);
			free(iolinkDestination);
			free(iolinkMidplane);

			rc = ll_get_data(mp, LL_BgMPGetNextIOLink, &iol);
			if (rc) {
				printf("Error: ll_get_data LL_BgMPGetFirstIOLink rc = %d\n",rc);
				exit(1);
			}
		} // get next midplane iolinks


		rc = ll_get_data(machine, LL_BgMachineGetNextMP, &mp);
		if (rc) {
			printf("Error: ll_get_data LL_BgMachineGetNextMP rc = %d\n",rc);
			exit(1);
		}

		printf("\n");
	} // get next midplane 

	// Free objects obtained from Negotiator
	ll_free_objs(queryObject);
	// Free query element
	ll_deallocate(queryObject);

	return rc;
}
